//+==================================================================+
//|                                                      Exp_10R.mq4 |
//|                             Copyright  2008,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+==================================================================+
#property copyright "Copyright  2008, Nikolay Kositsin"
#property link "farria@mail.redcom.ru"
//----+ +------------------------------------------------------------+
//---- INPUT PARAMETERS OF THE EXPERT ADVISOR FOR SETTING PENDING ORDERS
extern int    NewsWeekDay = 5;  // Day of the week for news release
//if the parameter is less that one or greater than five, any working day is used
extern int    NewsHour = 15;  // hour of new release
extern int    NewsMinute = 0;  // minute of news release
extern int    OrderLife = 30;  // Life of a pending order in minutes
//----+ +------------------------------------------------------------+
//---- INPUT PARAMETERS OF THE EXPERT ADVISOR FOR CLOSING POSITIONS
extern int    PosLife = 120;  // Life of an open position in minutes 
                                                //from the news release time
//----+ +------------------------------------------------------------+
//---- INPUT PARAMETERS OF THE EXPERT ADVISOR FOR BUY DEALS 
extern bool   Test_Up = true;//filter of direction of deal calculations
extern double Money_Management_Up = 0.1;
extern int    STOPLOSS_Up = 50;  // Stop Loss in points
extern int    TAKEPROFIT_Up = 100; // Take Profit in points
extern int    TRAILINGSTOP_Up = 0; // Trailing Stop in points
extern int    PriceLevel_Up =40; // difference between the current price and 
                          // the pending order triggering price
extern bool   ClosePos_Up = true; // permission to forcibly 
                                                 //close a position
//----+ +------------------------------------------------------------+
//---- INPUT PARAMETERS OF THE EXPERT ADVISOR FOR SELL DEALS 
extern bool   Test_Dn = true;//filter of direction of deal calculations
extern double Money_Management_Dn = 0.1;
extern int    STOPLOSS_Dn = 50;  // Stop Loss in points
extern int    TAKEPROFIT_Dn = 100; // Take Profit in points
extern int    TRAILINGSTOP_Dn = 0; // Trailing Stop in points
extern int    PriceLevel_Dn = 40; // difference between the current price and
 
                          // the pending order triggering price
extern bool   ClosePos_Dn = true; // permission to forcibly 
                                                 //close a position
//Version  January 20, 2009
//+X================================================================X+
//|                                                 Lite_EXPERT2.mqh |
//|                             Copyright  2009,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+X================================================================X+

//---- Declaring an external variable for slippage in points 
extern int Slippage_ = 3;   
                                
//---- Declaring a global variable for storing 
                            //the time of the last server call 
int LastTime;

//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+   
/*                                                                   |
 The Margin_Mode external variable determines the lot size | 
 calculation                                                                |
 0 - MM based on free margin                              |
 1 - MM based on the account balance                                  |
 2 - MM based on losses on free margin                     |
 3 - MM based on losses on the account balance                       |
 4 - minimum lot between 0 and 2                                     |
 5 - minimum lot between 1 and 3                                     |
 by default - MM based on free margin                   |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
 if Money_Management is negative, the trading function uses | 
 the absolute Money_Management value rounded to the nearest standard value |
 as the lot size.                      |
*/                                                                // |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+

//+X================================================================X+
//| OpenBuyStopOrder1_()                                             |
//+X================================================================X+
bool OpenBuyStopOrder1_
        (bool& Order_Signal, int MagicNumber, datetime TimeLevel, 
                             double Money_Management, int Margin_Mode, 
                                           int STOPLOSS, int TAKEPROFIT,
                                             int LEVEL, datetime Expiration)
 {
//---+
  if (!Order_Signal)
             return(true); 
  //---- Check if the minimum time interval has passed 
                                    //between two trading operations         
  if (TimeCurrent() < LastTime)
                          return(true);
  //----
  string Symb = Symbol();

  //---- Check if the time limit with respect to the previous deal has expired
  if (!TradeTimeLevelCheck(Symb, MagicNumber, TimeLevel))
                                                  return(true);
                                  
  //---- Check for a placed order or open position
             //with the magic number equal to the MagicNumber variable value
  if (Order_Select(Symb, MagicNumber))
                                 return(true);
                                 
  //---- Check correctness of pending orders
  if (STOPLOSS < 0)
   {
    Print("OpenBuyStopOrder1_(): Invalid Stop Loss!!!");
    return(true);
   }
  //---- 
  if (TAKEPROFIT < 0)
   {
    Print("OpenBuyStopOrder1_(): Invalid Take Profit!!!");
    return(true);
   }
  //---- 
  if (LEVEL < 0)
   {
    Print("OpenBuyStopOrder1_(): Invalid BuyStop order!!!");
    return(true);
   }
  //----  
  string OpderPrice;
  bool   Order_Select; 
  int    ticket, StLOSS, TkPROFIT, Level, ErrorCode;
  double Lot, OpenPrice, Stoploss, TakeProfit;                                                     
  //----         
  Level = StopCorrect(Symb, LEVEL, 0);
  //----
  OpenPrice = NormalizeDouble(Ask + Level * Point, Digits);
  if (OpenPrice == 0.0)
          return(false);
  //---- 
  if (STOPLOSS > 0) 
   {           
     StLOSS = StopCorrect(Symb, STOPLOSS, 0);
     if (StLOSS < 0)
          return(false);
     //----
     Stoploss = NormalizeDouble(OpenPrice - StLOSS * Point, Digits);
     if (Stoploss < 0)
           return(false);
     //---- 
   }
  else Stoploss = 0.0;
  //----   
  if (TAKEPROFIT > 0) 
   {           
     TkPROFIT = StopCorrect(Symb, TAKEPROFIT, 0);
     if (TkPROFIT < 0)
          return(false);
     //----
     TakeProfit = NormalizeDouble(OpenPrice + TkPROFIT * Point, Digits);
     if (TakeProfit < 0)
                return(false);
     //---- 
   }
  else TakeProfit = 0.0;
                                                                                                    
  //---+ LOT SIZE CALCULATION FOR OPENING A POSITION
  Lot = BuyLotCount(Money_Management, Margin_Mode, StLOSS);
  //----
  if (Lot == -1)
            return(false);  
  else 
   if (Lot == 0)
             return(true);          
  //---- 
  Print(StringConcatenate
         ("Set a ", Symb,
                  " pending BUY STOP order with the magic number ", MagicNumber));
  //---+ Set a BUY STOP order   
  ticket = OrderSend(Symb, OP_BUYSTOP, Lot, OpenPrice, 0, 
            Stoploss, TakeProfit, NULL, MagicNumber, Expiration, Blue); 
  
  //---- Calculation of the time interval between trading operations         
  CountLastTime(GetLastError());
  //----
  Order_Select = OrderSelect(ticket, SELECT_BY_TICKET);
  
  //---- Calculation of the time interval between trading operations         
  ReCountLastTime(GetLastError());
  //----
  if(ticket > 0)
   {
     if (Order_Select)
       {
         //---- Blocking the OpenBuyStopOrder1_() function
                            // from repeated performance of the operation
         if(TimeLevel < 0)
                   Order_Signal = false;
         //----
         if(TimeLevel > 0)
            GlobalVariableSet(GetTimeLevelName(Symb, MagicNumber), TimeLevel);
         //----   
         OpderPrice = DoubleToStr(OrderOpenPrice(), Digits);
         Print(StringConcatenate("Pending BUY STOP ",Symb, 
            " order with ticket #",  ticket, " and the magic number ", 
               OrderMagicNumber(), " has been set. The order triggering price ", OpderPrice,
                            ". The calculated price was ", DoubleToStr(OpenPrice, Digits)));
       }
     else
       Print(StringConcatenate("Failed to set a pending BUY STOP ", Symb, 
                               " order with the magic number ", MagicNumber, "!!!"));
   }
  else
   Print(StringConcatenate("Failed to set a pending BUY STOP ", Symb, 
                               " order with the magic number ", MagicNumber, "!!!"));
  //----      
  return(true);
//---+
 }
//+X================================================================X+
//| OpenSellStopOrder1_()                                            |
//+X================================================================X+
bool OpenSellStopOrder1_
        (bool& Order_Signal, int MagicNumber, datetime TimeLevel,
                            double Money_Management, int Margin_Mode,
                                           int STOPLOSS, int TAKEPROFIT,
                                            int LEVEL, datetime Expiration)
 {
//---+
  if (!Order_Signal)
           return(true); 
  //---- Check if the minimum time interval has passed 
                                    //between two trading operations         
  if (TimeCurrent() < LastTime)
                          return(true);
  //----
  string Symb = Symbol();

  //---- Check if the time limit with respect to the previous deal has expired
  if (!TradeTimeLevelCheck(Symb, MagicNumber, TimeLevel))
                                                  return(true);
                                  
  //---- Check for a placed order or open position
             //with the magic number equal to the MagicNumber variable value
  if (Order_Select(Symb, MagicNumber))
                                 return(true); 
                                 
  //---- Check correctness of pending orders
  if (STOPLOSS < 0)
   {
    Print("OpenSellStopOrder1_(): Invalid Stop Loss!!!");
    return(true);
   }
  //---- 
  if (TAKEPROFIT < 0)
   {
    Print("OpenSellStopOrder1_(): Invalid Take Profit!!!");
    return(true);
   }
  //---- 
  if (LEVEL < 0)
   {
    Print("OpenSellStopOrder1_(): Invalid SellStop order!!!");
    return(true);
   }
  //----
  string OpderPrice;
  bool   Order_Select; 
  int    ticket, StLOSS, TkPROFIT, Level, ErrorCode;
  double Lot, OpenPrice, Stoploss, TakeProfit;                                                                                                                      
  //---- 
  Level = StopCorrect(Symb, LEVEL, 0);        
  //----
  OpenPrice = NormalizeDouble(Bid - Level * Point, Digits);
  if (OpenPrice == 0.0)
               return(false);   
  //---- 
  if (STOPLOSS > 0) 
   {           
     StLOSS = StopCorrect(Symb, STOPLOSS, 0);
     if (StLOSS < 0)
          return(false);
     //----
     Stoploss = NormalizeDouble(OpenPrice + StLOSS * Point, Digits);
     if (Stoploss < 0)
           return(false);
     //---- 
   }
  else Stoploss = 0.0;
  //----   
  if (TAKEPROFIT > 0) 
   {           
     TkPROFIT = StopCorrect(Symb, TAKEPROFIT, 0);
     if (TkPROFIT < 0)
          return(false);
     //----
     TakeProfit = NormalizeDouble(OpenPrice - TkPROFIT * Point, Digits);
     if (TakeProfit < 0)
                return(false);
     //---- 
   }
  else TakeProfit = 0.0;
  
  //---+ LOT SIZE CALCULATION FOR OPENING A POSITION
  Lot = SellLotCount(Money_Management, Margin_Mode, StLOSS);
  //----
  if (Lot == -1)
            return(false);  
  else 
   if (Lot == 0)
             return(true);      
  //---- 
  Print(StringConcatenate
         ("Set a ", Symb,
                  " pending SELL STOP order with the magic number ", MagicNumber));
  //---+ Set a SELL STOP order 
  ticket = OrderSend(Symb, OP_SELLSTOP, Lot, OpenPrice, 0, 
            Stoploss, TakeProfit, NULL, MagicNumber, 0, Magenta);
             
  //---- Calculation of the time interval between trading operations         
  CountLastTime(GetLastError());
  //----
  Order_Select = OrderSelect(ticket, SELECT_BY_TICKET);
  
  //---- Calculation of the time interval between trading operations         
  ReCountLastTime(GetLastError());
  //----
  if(ticket > 0)
   {
     if (Order_Select)
       { 
         //---- Blocking the OpenSellStopOrder1_() function
                            // from repeated performance of the operation
         if(TimeLevel < 0)
                   Order_Signal = false;
         //----
         if(TimeLevel > 0)
            GlobalVariableSet(GetTimeLevelName(Symb, MagicNumber), TimeLevel);
         //---- 
         OpderPrice = DoubleToStr(OrderOpenPrice(), Digits);
         Print(StringConcatenate("Pending SELL STOP ",Symb, 
               " order with ticket #",  ticket, " and the magic number ", 
                 OrderMagicNumber(), " has been set. The order triggering price is ",OpderPrice, 
                              ". The calculated price was ", DoubleToStr(OpenPrice, Digits)));
       }
     else   
       Print(StringConcatenate("Failed to set a pending SELL STOP ", Symb, 
                               " order with the magic number ", MagicNumber, "!!!"));
   }
  else   
   Print(StringConcatenate("Failed to set a pending SELL STOP ", Symb, 
                               " order with the magic number ", MagicNumber, "!!!"));
  //----       
  return(true);
//---+  
 }
//+X================================================================X+
//| CloseBuyOrder1_() function                                       |
//+X================================================================X+

bool CloseBuyOrder1_(bool& CloseStop, int MagicNumber)
  {
//----+
   if (!CloseStop)
          return(true);
   //---- Check if the minimum time interval has passed 
                                    //between two trading operations  
   if (TimeCurrent() < LastTime)
                          return(true); 
   //----                
   int total = OrdersTotal(); 
   if (total == 0)
            return(true);
   //----
   double priceClose;
   bool   Close_Order;
   color  Order_Color;
   //---- 
   int    FreezeLevelT, FreezeLevelS;
   int    ticket, DIGITS, FREEZELEVEL;
   string ClosePrice, Symb, Order_Type;
   
   //---- search for an open position with the required magic number 
   if (!OrderSelect_(Symbol(), OP_BUY, MagicNumber, MODE_TRADES))
                                                          return(true);
   //---- 
   ticket = OrderTicket(); 
   Symb = OrderSymbol();
   //---- 
   DIGITS = MarketInfo(OrderSymbol(), MODE_DIGITS); 
   if (DIGITS == 0)
             return(false);
                
   //----+ Getting values to delete the order
   priceClose = MarketInfo(Symb, MODE_BID);
   if (priceClose == 0)
                return(false);
   Order_Type =" BUY";
   Order_Color = Red;
 
   //----+ Check the order for freezing 
   FREEZELEVEL = MarketInfo(OrderSymbol(), MODE_FREEZELEVEL);
   FreezeLevelT = MathAbs(priceClose - OrderTakeProfit()) / Point;
   FreezeLevelS = MathAbs(priceClose - OrderStopLoss()) / Point;
   //----
   if (FreezeLevelT <= FREEZELEVEL
        || FreezeLevelS <= FREEZELEVEL)
                               return(true);
   
   //----+  Close the open position                                 
   Print(StringConcatenate("Close ",
                 Symb, Order_Type, " order with ticket #", ticket,
                                   " and the magic number ", MagicNumber));
   //---- 
   Close_Order = OrderClose(ticket, OrderLots(), priceClose, Slippage_, Order_Color);
   //---- Calculation of the time interval between trading operations         
   CountLastTime(GetLastError());
   //----                                        
   if(Close_Order)
     {
       CloseStop = false;
       ClosePrice = DoubleToStr(OrderClosePrice(), DIGITS);  
       Print(StringConcatenate(Symb, Order_Type, " order with ticket #",
                       ticket, " and magic number ", OrderMagicNumber(), 
                                             " closed at the price ", ClosePrice));
     }
   else Print(StringConcatenate("Failed to close the ", Symb, 
          Order_Type," order with ticket #", ticket, " and the magic number ", 
                                                         MagicNumber, "!!!"));
   //----   
   return(true);
//----+
  }  
//+X================================================================X+
//| CloseSellOrder1_() function                                      |
//+X================================================================X+

bool CloseSellOrder1_(bool& CloseStop, int MagicNumber)
  {
//----+
   if (!CloseStop)
          return(true);
   //---- Check if the minimum time interval has passed 
                                    //between two trading operations  
   if (TimeCurrent() < LastTime)
                          return(true); 
   //----                
   int total = OrdersTotal(); 
   if (total == 0)
            return(true);
   //----
   double priceClose;
   bool   Close_Order;
   color  Order_Color;
   //---- 
   int    FreezeLevelT, FreezeLevelS;
   int    ticket, DIGITS, FREEZELEVEL;
   string ClosePrice, Symb, Order_Type;
   
   //---- search for an open position with the required magic number 
   if (!OrderSelect_(Symbol(), OP_SELL, MagicNumber, MODE_TRADES))
                                                          return(true);
   //---- 
   ticket = OrderTicket(); 
   Symb = OrderSymbol();
   //---- 
   DIGITS = MarketInfo(OrderSymbol(), MODE_DIGITS); 
   if (DIGITS == 0)
             return(false);
                
   //----+ Getting values to delete the order
   priceClose = MarketInfo(Symb, MODE_ASK);
   if (priceClose == 0)
                return(false);
   Order_Type =" SELL";
   Order_Color = Lime;

   //----+ Check the order for freezing 
   FREEZELEVEL = MarketInfo(OrderSymbol(), MODE_FREEZELEVEL);
   FreezeLevelT = MathAbs(priceClose - OrderTakeProfit()) / Point;
   FreezeLevelS = MathAbs(priceClose - OrderStopLoss()) / Point;
   //----
   if (FreezeLevelT <= FREEZELEVEL
         || FreezeLevelS <= FREEZELEVEL)
                               return(true);
   
   //----+  Close the open position                                 
   Print(StringConcatenate("Close ",
                 Symb, Order_Type, " order with ticket #", ticket,
                                   " and the magic number ", MagicNumber));
   //---- 
   Close_Order = OrderClose(ticket, OrderLots(), priceClose, Slippage_, Order_Color);
   
   //---- Calculation of the time interval between trading operations         
   CountLastTime(GetLastError());
   //----                                        
   if(Close_Order)
     {
       CloseStop = false;
       ClosePrice = DoubleToStr(OrderClosePrice(), DIGITS);  
       Print(StringConcatenate(Symb, Order_Type, " order with ticket #",
                       ticket, " and magic number ", OrderMagicNumber(), 
                                             " closed at the price ", ClosePrice));
     }
   else Print(StringConcatenate("Failed to close the ", Symb, 
          Order_Type," order with ticket #", ticket, " and the magic number ", 
                                                         MagicNumber, "!!!"));
   //----   
   return(true);
//----+
  } 
//+X================================================================X+
//| Make_BuyTrailingStop_() function                                 |
//+X================================================================X+
bool Make_BuyTrailingStop_
                (bool& TrailSignal, int MagicNumber, 
                          datetime TrailTimeLevel, int TRAILINGSTOP)
 {
//----+
  if(TRAILINGSTOP <= 0 || !TrailSignal)
                                return(true);
  //---- Check if the minimum time interval has passed 
                                  //between two trading operations         
  if (TimeCurrent() < LastTime)
                          return(true);
   
  //----
  string symb = Symbol();

  //---- Check if the time limit with respect to the previous trailing stop has expired
  if (!TrailTimeLevelCheck(symb, MagicNumber, TrailTimeLevel))
                                                     return(true);
  //----    
  if (OrdersTotal() == 0)
                 return(true);
              
  //---- Check for an open position 
        //with the magic number equal to the MagicNumber variable value  
  if (!OrderSelect_(symb, OP_BUY, MagicNumber, MODE_TRADES))
                                                    return(true);            
  //---- 
  bool Order_Modify;          
  color Order_Color;
  //----
  double NewStopLoss, bid, point, TRStop, price;
  string NEWSTOPLOSS, Order_Type, PriceName;
  //----
  int FREEZELEVEL, ticket, digits;
  int FreezeLevelT, FreezeLevelS;      
  //----
  symb = OrderSymbol();           
  ticket = OrderTicket(); 
  //----
  symb = OrderSymbol();          
  ticket = OrderTicket(); 
  //----
  digits = MarketInfo(symb, MODE_DIGITS); 
  if (digits == 0)
            return(false);
  //----
  bid = NormalizeDouble(MarketInfo(symb, MODE_BID), digits); 
  if (bid == 0)
          return(false);
  //----
  point = MarketInfo(symb, MODE_POINT); 
  if (point == 0)
           return(false); 
  //----
  TRStop = StopCorrect(symb, TRAILINGSTOP, 0) * point;
  if (TRStop < 0)
           return(false);

  //---+ Getting values for order modification
  if(bid - OrderStopLoss() > TRStop)
	{
	 price = bid;
	 NewStopLoss = NormalizeDouble(bid - TRStop, digits);
	 Order_Type =" BUY";
	 Order_Color = Lime;
	 PriceName = StringConcatenate(" BID = ", 
									DoubleToStr(bid, digits));
	}
  else return(true);
  
  //---- Check if the Stop Loss value is sufficient for modification
  if (NewStopLoss <= OrderStopLoss())
                             return(true);
  		
  //---+ Check the position for freezing 
  FREEZELEVEL = MarketInfo(OrderSymbol(), MODE_FREEZELEVEL);
  FreezeLevelT = MathAbs(price - OrderTakeProfit()) / point;
  FreezeLevelS = MathAbs(price - OrderStopLoss()) / point;
  //----
  if (FreezeLevelT <= FREEZELEVEL 
      || FreezeLevelS <= FREEZELEVEL)
							    	return(true);
										
  //---+ Modify the order 
  if(OrderCloseTime() != 0)
							return(true);
  Print(StringConcatenate("Move the Stop Loss using the Trailing Stop of ", 
            									        TRAILINGSTOP, " points"));
  Print(StringConcatenate("Modify the Stop Loss. ",
						 symb, Order_Type, " order with ticket #",ticket, 
										     " and the magic number ", MagicNumber)); 
  //----
  Order_Modify = OrderModify(ticket, OrderOpenPrice(), NewStopLoss, 
       											OrderTakeProfit(), 0, Order_Color);
  //---- Calculation of the time interval between trading operations
  CountLastTime(GetLastError());
  //----                                   
  if (Order_Modify)
   {
    //---- Blocking the Make_BuyTrailingStop_() function
                                   // from repeated performance of the operation
    if(TrailTimeLevel < 0)
              TrailSignal = false;
    //----
    if(TrailTimeLevel > 0)
       GlobalVariableSet(GetTrailTimeLevelName(symb, MagicNumber), TrailTimeLevel);
    //----
    Print(StringConcatenate(symb, Order_Type, " order with ticket #", 
		ticket, " and the magic number ", OrderMagicNumber(), " has been modified"));
	 //----              
	 NEWSTOPLOSS = DoubleToStr(OrderStopLoss(), digits);  
	 Print(StringConcatenate("New Stop Loss of the order ", 
							NEWSTOPLOSS, ". The calculated Stop Loss ", 
								DoubleToStr(NewStopLoss, digits), ".", PriceName)); 
   }
  else Print("Failed to modify ", symb, Order_Type, 
						" order with ticket #",ticket, " and the magic number ", 
																		OrderMagicNumber());
  //----    
  return(true);   
//----+
  } 
//+X================================================================X+
//| Make_SellTrailingStop_() function                                |
//+X================================================================X+
bool Make_SellTrailingStop_
                (bool& TrailSignal, int MagicNumber, 
                          datetime TrailTimeLevel, int TRAILINGSTOP)
 {
//----+
  if(TRAILINGSTOP <= 0)
                     return(true); 
  //---- Check if the minimum time interval has passed 
                                  //between two trading operations         
  if (TimeCurrent() < LastTime)
                          return(true); 
  //----
  string symb = Symbol();

  //---- Check if the time limit with respect to the previous trailing stop has expired
  if (!TrailTimeLevelCheck(symb, MagicNumber, TrailTimeLevel))
                                                     return(true);
  //----    
  if (OrdersTotal() == 0)
                 return(true);
              
  //---- Check for an open position 
        //with the magic number equal to the MagicNumber variable value  
  if (!OrderSelect_(symb, OP_SELL, MagicNumber, MODE_TRADES))
                                                         return(true);            
  //---- 
  bool Order_Modify;          
  color Order_Color;
  //----
  double NewStopLoss, ask, point, TRStop, price;
  string NEWSTOPLOSS, Order_Type, PriceName;
  //----
  int FREEZELEVEL, ticket, digits;
  int FreezeLevelT, FreezeLevelS;      
  //----
  symb = OrderSymbol();           
  ticket = OrderTicket(); 
  //----
  digits = MarketInfo(symb, MODE_DIGITS); 
  if (digits == 0)
            return(false);
  //----
  ask = NormalizeDouble(MarketInfo(symb, MODE_ASK), digits); 
  if (ask == 0)
          return(false);
  //----
  point = MarketInfo(symb, MODE_POINT); 
  if (point == 0)
           return(false); 
  //----
  TRStop = StopCorrect(symb, TRAILINGSTOP, 0) * point;
  if (TRStop < 0)
           return(false);

  //---+ Getting values for order modification
  if(OrderStopLoss() - ask > TRStop)
	{
	 price = ask;
	 NewStopLoss = NormalizeDouble(ask + TRStop, digits);
	 Order_Type =" SELL";
	 Order_Color = Red;
	 PriceName = StringConcatenate(" ASK = ", 
									DoubleToStr(ask, digits));
	}
  else return(true);
  
  //---- Check if the Stop Loss value is sufficient for modification
  if (NewStopLoss >= OrderStopLoss())
                             return(true);
  	
  //---+ Check the position for freezing 
  FREEZELEVEL = MarketInfo(OrderSymbol(), MODE_FREEZELEVEL);
  FreezeLevelT = MathAbs(price - OrderTakeProfit()) / point;
  FreezeLevelS = MathAbs(price - OrderStopLoss()) / point;
  //----
  if (FreezeLevelT <= FREEZELEVEL 
      || FreezeLevelS <= FREEZELEVEL)
							    	return(true);
										
  //---+ Modify the order 
  if(OrderCloseTime() != 0)
							return(true);
  Print(StringConcatenate("Move the Stop Loss using the Trailing Stop of ", 
            											TRAILINGSTOP, " points"));
  Print(StringConcatenate("Modify the Stop Loss. ",
						symb, Order_Type, " order with ticket #",ticket, 
											" and the magic number ", MagicNumber)); 
  //----
  Order_Modify = OrderModify(ticket, OrderOpenPrice(), NewStopLoss, 
       											OrderTakeProfit(), 0, Order_Color);
  //---- Calculation of the time interval between trading operations
  CountLastTime(GetLastError());
  //----                                   
  if(Order_Modify)
   {
    //---- Blocking the Make_SellTrailingStop_() function 
                                   // from repeated performance of the operation
    if(TrailTimeLevel < 0)
              TrailSignal = false;
    //----
    if(TrailTimeLevel > 0)
       GlobalVariableSet(GetTrailTimeLevelName(symb, MagicNumber), TrailTimeLevel);
    //----
    Print(StringConcatenate(symb, Order_Type, " order with ticket #", 
		ticket, " and the magic number ", OrderMagicNumber(), " has been modified"));
	 //----              
    NEWSTOPLOSS = DoubleToStr(OrderStopLoss(), digits);  
	 Print(StringConcatenate("New Stop Loss of the order ", 
							NEWSTOPLOSS, ". The calculated Stop Loss ", 
								DoubleToStr(NewStopLoss, digits), ".", PriceName)); 
   }
  else Print("Failed to modify ", symb, Order_Type, 
						" order with ticket #",ticket, " and the magic number ", 
																		OrderMagicNumber());
  //----    
  return(true);   
//----+
  }
//+X================================================================X+
//| TimeLevelGlobalVariableDel() functions                           |
//+X================================================================X+
void TimeLevelGlobalVariableDel(string symbol, int MagicNumber)
  {
//---+
   if (IsTesting() || IsOptimization())
    {
     GlobalVariableDel(GetTimeLevelName       (symbol, MagicNumber));
     GlobalVariableDel(GetTrailTimeLevelName  (symbol, MagicNumber));                  
    }
//---+ 
  }
//+X================================================================X+
//| OrderSelect_() functions                                         |
//+X================================================================X+
bool OrderSelect_(string symbol, int cmd, int MagicNumber, int pool)
  {
//---+
   //----                       
   int total;
   //----
   if (pool == MODE_TRADES)
         total = OrdersTotal();
   else 
    if (pool == MODE_HISTORY)           
         total = OrdersHistoryTotal();
  
   //---- Check for an open position 
          //with the magic number equal to the MagicNumber variable value
   for(int pos = total - 1; pos >= 0; pos--)     
      if (OrderSelect(pos, SELECT_BY_POS, pool))
       if (OrderType() == cmd)
        if (OrderSymbol() == symbol)
         if (OrderMagicNumber() == MagicNumber)
                                          return(true); 
   return(false); 
//---+ 
  }
//+X================================================================X+
//| Order_Select() functions                                         |
//+X================================================================X+
bool Order_Select(string symbol, int MagicNumber)
  {
//---+
   //----                       
   int total = OrdersTotal();
  
   //---- Check for an open position or order 
          //with the magic number equal to the MagicNumber variable value
   for(int pos = total - 1; pos >= 0; pos--)     
      if (OrderSelect(pos, SELECT_BY_POS, MODE_TRADES))
        if (OrderSymbol() == symbol)
         if (OrderMagicNumber() == MagicNumber)
                                          return(true); 
   return(false); 
//---+ 
  }
//+X================================================================X+
//| CountLastTime() function                                         |
//+X================================================================X+
void CountLastTime(int Error)
  {
//---+
   int TimeCurr = TimeCurrent();
   //---
   switch(Error)
       {
        case 000: LastTime = TimeCurr + 000; break;
        case 001: LastTime = TimeCurr + 060; break;
        case 002: LastTime = TimeCurr + 300; break;
        case 003: LastTime = TimeCurr + 300; break;                                                                  
        case 004: LastTime = TimeCurr + 180; break; 
        case 005: LastTime = TimeCurr + 000; break;
        case 006: LastTime = TimeCurr + 006; break;
        case 008: LastTime = TimeCurr + 011; break;
        case 064: LastTime = TimeCurr + 300; break;
        case 065: LastTime = TimeCurr + 180; break;                                                     
        case 128: LastTime = TimeCurr + 065; break; 
        case 129: LastTime = TimeCurr + 008; break; 
        case 130: LastTime = TimeCurr + 008; break;  
        case 131: LastTime = TimeCurr + 300; break;
        case 132: LastTime = TimeCurr + 180; break;
        case 133: LastTime = TimeCurr + 300; break;
        case 134: LastTime = TimeCurr + 008; break; 
        case 135: LastTime = TimeCurr + 000; break;
        case 136: LastTime = TimeCurr + 008; break; 
        case 138: LastTime = TimeCurr + 000; break;
        case 139: LastTime = TimeCurr + 011; break;
        case 140: LastTime = TimeCurr + 011; break; 
        case 141: LastTime = TimeCurr + 011; break;
        case 142: LastTime = TimeCurr + 065; break; 
        case 143: LastTime = TimeCurr + 065; break; 
        case 144: LastTime = TimeCurr + 011; break; 
        case 145: LastTime = TimeCurr + 017; break;  
        case 146: LastTime = TimeCurr + 300; break;
        case 147: LastTime = TimeCurr + 011; break;
        case 148: LastTime = TimeCurr + 300; break;
        default:  LastTime = TimeCurr + 000; break;
       }
//---+
  }
//+X================================================================X+
//| ReCountLastTime() function                                       |
//+X================================================================X+
void ReCountLastTime(int Error)
  {
//---+
   //---
   switch(Error)
       {
        case 000: LastTime += 000; break;
        case 001: LastTime += 060; break;
        case 002: LastTime += 300; break;
        case 003: LastTime += 300; break;                                                                  
        case 004: LastTime += 180; break; 
        case 005: LastTime += 000; break;
        case 006: LastTime += 006; break;
        case 008: LastTime += 011; break;
        case 064: LastTime += 300; break;
        case 065: LastTime += 180; break;                                                     
        case 128: LastTime += 065; break; 
        case 129: LastTime += 007; break; 
        case 130: LastTime += 007; break;  
        case 131: LastTime += 300; break;
        case 132: LastTime += 180; break;
        case 133: LastTime += 300; break;
        case 134: LastTime += 007; break; 
        case 135: LastTime += 000; break;
        case 136: LastTime += 007; break; 
        case 138: LastTime += 000; break;
        case 139: LastTime += 011; break;
        case 140: LastTime += 011; break; 
        case 141: LastTime += 011; break;
        case 142: LastTime += 065; break; 
        case 143: LastTime += 065; break; 
        case 144: LastTime += 011; break; 
        case 145: LastTime += 017; break;  
        case 146: LastTime += 300; break;
        case 147: LastTime += 011; break;
        case 148: LastTime += 300; break;
        default:  LastTime += 000; break;
       }
//---+
  }
//+X================================================================X+
//| GetFreeMargin() function                                         |
//+X================================================================X+
double GetFreeMargin()
  {
//---+
   switch(AccountFreeMarginMode())
    {     
     case 0: return(AccountFreeMargin() + AccountProfit());
     case 1: return(AccountFreeMargin());
     case 2: if (AccountProfit() > 0) 
                  return(AccountFreeMargin());
             else return(AccountFreeMargin() - AccountProfit());
     case 3: if (AccountProfit() < 0) 
                  return(AccountFreeMargin());
             else return(AccountFreeMargin() - AccountProfit());
    }
//---+
  } 
//+X================================================================X+
//| StopCorrect() function                                           |
//+X================================================================X+
int StopCorrect(string symbol, int Stop, int slippage_)
  {
//---+
   int CorrStop, Extrem_Stop;
   //---
   Extrem_Stop = MarketInfo(symbol, MODE_STOPLEVEL) + slippage_;
   //---             
   if(Stop < Extrem_Stop)
        CorrStop = Extrem_Stop;
   else CorrStop = Stop;
   //---   
   return(CorrStop);
//---+
  }
//+X================================================================X+
//| GetTimeLevelName() function                                      |
//+X================================================================X+
string GetTimeLevelName(string symbol, int MagicNumber)
  {
//---+
   string G_Name_;
   static string G_Name_0;
   static bool SecondStart;
   //---- Initialization to zero
   if (!SecondStart)
    {
     G_Name_0 = StringConcatenate
             ("TimeLevel", "_", AccountNumber(), "_");
     //----  
     if (IsTesting() || IsOptimization())
          G_Name_0 = StringConcatenate(G_Name_0, "_Test_");
     //----              
     SecondStart = true;
    }
   //----
   G_Name_ = StringConcatenate(G_Name_0, symbol, "_", MagicNumber);   
   //----   
   return(G_Name_);
//---+
  }
//+X================================================================X+
//| GetTrailTimeLevelName() function                                 |
//+X================================================================X+
string GetTrailTimeLevelName(string symbol, int MagicNumber)
  {
//---+
   string G_Name_;
   static string G_Name_0;
   static bool SecondStart;
   //---- Initialization to zero
   if (!SecondStart)
    {
     G_Name_0 = StringConcatenate
             ("TrailTimeLevel", "_", AccountNumber(), "_");
     //----  
     if (IsTesting() || IsOptimization())
          G_Name_0 = StringConcatenate(G_Name_0, "_Test_");
     //----              
     SecondStart = true;
    }
   //----
   G_Name_ = StringConcatenate(G_Name_0, symbol, "_", MagicNumber);   
   //----   
   return(G_Name_);
//---+
  }
//+X================================================================X+
//| TradeTimeLevelCheck() function                                   |
//+X================================================================X+
bool TradeTimeLevelCheck
             (string symbol, int MagicNumber, datetime TradeTimeLevel)
  {
//---+
   //----
  if (TradeTimeLevel > 0)
   {
    //---- Getting the name of a global 
          //variable for storing the time limit
    string G_Name_ = GetTimeLevelName(symbol, MagicNumber);
  
    //---- Check if the time limit with respect to the previous deal has expired 
    if (TimeCurrent() < GlobalVariableGet(G_Name_))
                                            return(false); 
    } 
   return(true);
//---+
  }
//+X================================================================X+
//| TrailTimeLevelCheck() function                                   |
//+X================================================================X+
bool TrailTimeLevelCheck
             (string symbol, int MagicNumber, datetime TrailTimeLevel)
  {
//---+
   //----
  if (TrailTimeLevel > 0)
   {
    //---- Getting the name of a global 
          //variable for storing the time limit
    string G_Name_ = GetTrailTimeLevelName(symbol, MagicNumber);
  
    //---- Check if the time limit with respect to the previous deal has expired 
    if (TimeCurrent() < GlobalVariableGet(G_Name_))
                                            return(false); 
    } 
   return(true);
//---+
  }
//+X================================================================X+
//| BuyLotCount_() function                                          |  
//+X================================================================X+
/*                                                                   |
 The Margin_Mode external variable determines the lot size | 
 calculation                                                                |
 0 - MM based on free margin                              |
 1 - MM based on the account balance                                  |
 2 - MM based on losses on free margin                     |
 3 - MM based on losses on the account balance                       |
 by default - MM based on free margin                   |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
 if Money_Management is negative, the trading function uses | 
 the absolute Money_Management value rounded to the nearest standard value |
 as the lot size.                      |
*/                                                                // |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
double BuyLotCount_
           (double Money_Management, int Margin_Mode, int STOPLOSS)
  {
//---+
  string Symb = Symbol(); 
  int    StLOSS;
  double LOTSTEP, MINLOT, MAXLOT, MARGINREQUIRED, TICKVALUE;
  double FreeMargin, LotVel, Lot;                                             

  //---1+ LOT SIZE CALCULATION FOR OPENING A POSITION
  if (Money_Management < 0)
       LotVel = MathAbs(Money_Management);
  else      
   switch(Margin_Mode)
    {
     //---+ Lot calculations based on free margin
     case  0:       
      MARGINREQUIRED = MarketInfo(Symb, MODE_MARGINREQUIRED);
      if (MARGINREQUIRED == 0.0)     
                           return(-1);             
      //----              
      LotVel = GetFreeMargin()
                * Money_Management / MARGINREQUIRED;               
      break;
  
    //---+ Lot calculation based on the account balance
    case  1:     
     MARGINREQUIRED = MarketInfo(Symb, MODE_MARGINREQUIRED);
     if (MARGINREQUIRED == 0.0)    
                       return(-1);          
     //----                
     LotVel = AccountBalance()
                  * Money_Management / MARGINREQUIRED;            
     break;
  
    //---+ Lot calculation based on losses on free margin             
    case  2:
     if (STOPLOSS <= 0)
      {
       Print("BuyLotCount(): Invalid Stop Loss!!!");
       STOPLOSS = 0;
      }
     //---- 
     StLOSS = StopCorrect(Symb, STOPLOSS, Slippage_);
     if (StLOSS <= 0)     
                return(-1);
     //----
     TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
     if (TICKVALUE == 0.0)  
                   return(-1);            
     //----               
     LotVel = GetFreeMargin() *  MathAbs(Money_Management)
                                     / (TICKVALUE * StLOSS);
     break;
   
    //---+ Lot calculation based on losses on the account balance
    case  3:
     if (STOPLOSS <= 0)
      {
       Print("BuyLotCount(): Invalid Stop Loss!!!");
       STOPLOSS = 0;
      }
     //---- 
     StLOSS = StopCorrect(Symb, STOPLOSS, Slippage_);
     if (StLOSS <= 0)    
              return(-1);
     //----
     TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
     if (TICKVALUE == 0.0)     
                    return(-1);            
     //----               
     LotVel = AccountBalance()
                *  MathAbs(Money_Management) 
                               / (TICKVALUE * StLOSS);
     break;
  
    //---+ Lot calculation based on free margin by default
    default:      
     MARGINREQUIRED = MarketInfo(Symb, MODE_MARGINREQUIRED);
     if (MARGINREQUIRED == 0.0)    
                        return(-1);              
     //----               
     LotVel = GetFreeMargin()
               * Money_Management / MARGINREQUIRED;         
    }
   //---1+   
   
  LOTSTEP = MarketInfo(Symb, MODE_LOTSTEP);
  if (LOTSTEP <= 0)    
           return(-1);
           
  //---- lot size normalization to the nearest standard value 
  Lot = LOTSTEP * MathFloor(LotVel / LOTSTEP);  
  
  //---+ check the lot for the minimum allowable value
  MINLOT = MarketInfo(Symb, MODE_MINLOT);
  if (MINLOT < 0)    
           return(-1);
  //----        
  if (Lot < MINLOT)  
            return(0);        
          
  //---+ check the lot for the maximum allowable value
  MAXLOT = MarketInfo(Symb, MODE_MAXLOT);
  if (MAXLOT < 0)    
           return(-1);         
  //----
  if (Lot > MAXLOT)
          Lot = MAXLOT;
  //----              
  return(Lot);        
//---+
  }   
//+X================================================================X+
//| SellLotCount_() function                                         |
//+X================================================================X+
/*                                                                   |
 The Margin_Mode external variable determines the lot size | 
 calculation                                                                |
 0 - MM based on free margin                              |
 1 - MM based on the account balance                                  |
 2 - MM based on losses on free margin                     |
 3 - MM based on losses on the account balance                       |
 by default - MM based on free margin                   |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
 if Money_Management is negative, the trading function uses | 
 the absolute Money_Management value rounded to the nearest standard value |
 as the lot size.                      |
*/                                                                // |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
double SellLotCount_
            (double Money_Management, int Margin_Mode, int STOPLOSS)
  {
//---+
  string Symb = Symbol(); 
  int    SPREAD, StLOSS;
  double LOTSTEP, MINLOT, MAXLOT, MARGINREQUIRED;
  double LotVel, Lot, TICKVALUE, TICKSIZE;                                               
  
  //---1+ LOT SIZE CALCULATION FOR OPENING A POSITION
  if (Money_Management < 0)
       LotVel = MathAbs(Money_Management);
  else      
   switch(Margin_Mode)
    {
     //---+ Lot calculations based on free margin
     case  0:     
      MARGINREQUIRED = MarketInfo(Symb, MODE_MARGINREQUIRED);
      if (MARGINREQUIRED == 0.0)    
                           return(-1);           
      //---
      SPREAD = MarketInfo(Symb, MODE_SPREAD);
      //---
      TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
      //---
      LotVel = GetFreeMargin() * Money_Management / 
                   (MARGINREQUIRED - (SPREAD * TICKVALUE));                  
      break;
  
    //---+ Lot calculation based on the account balance
    case  1:      
     MARGINREQUIRED = MarketInfo(Symb, MODE_MARGINREQUIRED);
     if (MARGINREQUIRED == 0.0)    
                        return(-1);          
     //---
     SPREAD = MarketInfo(Symb, MODE_SPREAD);
     //---
     TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
     //---
     LotVel = AccountBalance() * Money_Management / 
                   (MARGINREQUIRED - (SPREAD * TICKVALUE));                  
     break;
 
    //---+ Lot calculation based on losses on free margin
    case  2:
     if (STOPLOSS <= 0)
      {
       Print("SellLotCount(): Invalid Stop Loss!!!");
       STOPLOSS = 0;
      }
     //---- 
     StLOSS = StopCorrect(Symb, STOPLOSS, Slippage_);
     if (StLOSS <= 0)    
                return(-1);
     //----  
     TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
     if (TICKVALUE == 0.0)   
                    return(-1);          
     //---
     SPREAD = MarketInfo(Symb, MODE_SPREAD);
     //---              
     LotVel = GetFreeMargin()
                * MathAbs(Money_Management) 
                      / (TICKVALUE * (StLOSS + SPREAD));
     break;
   
    //---+ Lot calculation based on losses on the account balance  
    case  3:
     if (STOPLOSS <= 0)
      {
       Print("SellLotCount(): Invalid Stop Loss!!!");
       STOPLOSS = 0;
      }
     //---- 
     StLOSS = StopCorrect(Symb, STOPLOSS, Slippage_);
     if (StLOSS <= 0)    
               return(-1);
     //----  
     TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
     if (TICKVALUE == 0.0)   
                    return(-1);            
     //---
     SPREAD = MarketInfo(Symb, MODE_SPREAD);
     //---              
     LotVel = AccountBalance()
               * MathAbs(Money_Management) 
                      / (TICKVALUE * (StLOSS + SPREAD));
     break;                                                                                
   
    //---+ Lot calculation based on free margin by default
    default:        
     MARGINREQUIRED = MarketInfo(Symb, MODE_MARGINREQUIRED);
     if (MARGINREQUIRED == 0.0)     
                        return(-1);             
     //---
     SPREAD = MarketInfo(Symb, MODE_SPREAD);
     //---
     TICKVALUE = MarketInfo(Symb, MODE_TICKVALUE);
     //---
     LotVel = GetFreeMargin() * Money_Management / 
                  (MARGINREQUIRED - (SPREAD * TICKVALUE));                   
    }
   
  //----  
  LOTSTEP = MarketInfo(Symb, MODE_LOTSTEP); 
  if (LOTSTEP <= 0)    
           return(-1);
             
  //---- lot size normalization to the nearest standard value   
  Lot = LOTSTEP * MathFloor(LotVel / LOTSTEP);
  
  //---+ check the lot for the minimum allowable value
  MINLOT = MarketInfo(Symb, MODE_MINLOT);
  if (MINLOT < 0)    
           return(-1);            
   
  if (Lot < MINLOT)   
            return(0);           
          
  //---+ check the lot for the maximum allowable value
  MAXLOT = MarketInfo(Symb, MODE_MAXLOT);
  if (MAXLOT < 0)    
        return(-1);           
   
  if (Lot > MAXLOT)
          Lot = MAXLOT;
  //----       
  return(Lot);
//---+
  }
//+X================================================================X+
//| BuyLotCount() function                                           |  
//+X================================================================X+
/*                                                                   |
 The Margin_Mode external variable determines the lot size | 
 calculation                                                                |
 0 - MM based on free margin                              |
 1 - MM based on the account balance                                  |
 2 - MM based on losses on free margin                     |
 3 - MM based on losses on the account balance                       |
 4 - minimum lot between 0 and 2                                     |
 5 - minimum lot between 1 and 3                                     |
 by default - MM based on free margin                   |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
 if Money_Management is negative, the trading function uses | 
 the absolute Money_Management value rounded to the nearest standard value |
 as the lot size.                      |
*/                                                                // |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
double BuyLotCount
      (double Money_Management, int Margin_Mode, int STOPLOSS)
  {
//---+
   if (Money_Management < 0)
                 return(BuyLotCount_(Money_Management, 0, 0));
   //----
   double LotA, LotB;
   int MarModeA, MarModeB;
   //----
   switch(Margin_Mode)
    {
     case 4: 
      MarModeA = 0;
      MarModeB = 2;
      break;      
      //----
     case 5:
      MarModeA = 1;
      MarModeB = 3;
      break;
      //----                                                                 
     default: return(BuyLotCount_(Money_Management, 
                                      Margin_Mode, STOPLOSS));
    }
    
    LotA = BuyLotCount_(Money_Management, MarModeA, STOPLOSS);
    if (LotA == -1)
              return(-1);
    //----          
    LotB = BuyLotCount_(Money_Management, MarModeB, STOPLOSS);
    if (LotB == -1)
              return(-1);
   //----              
   if (LotA < LotB)
        return(LotA);
   else return(LotB);         
//---+
  }
//+X================================================================X+
//| SellLotCount() function                                          |  
//+X================================================================X+   
/*                                                                   |
 The Margin_Mode external variable determines the lot size | 
 calculation                                                                |
 0 - MM based on free margin                              |
 1 - MM based on the account balance                                  |
 2 - MM based on losses on free margin                     |
 3 - MM based on losses on the account balance                       |
 4 - minimum lot between 0 and 2                                     |
 5 - minimum lot between 1 and 3                                     |
 by default - MM based on free margin                   |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
 if Money_Management is negative, the trading function uses | 
 the absolute Money_Management value rounded to the nearest standard value |
 as the lot size.                      |
*/                                                                // |
//+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+
double SellLotCount
      (double Money_Management, int Margin_Mode, int STOPLOSS)
  {
//---+
   if (Money_Management < 0)
                return(SellLotCount_(Money_Management, 0, 0));
   //----
   double LotA, LotB;
   int MarModeA, MarModeB;
   //----
   switch(Margin_Mode)
    {
     case 4: 
      MarModeA = 0;
      MarModeB = 2;
      break;      
      //----
     case 5:
      MarModeA = 1;
      MarModeB = 3;
      break;
      //----                                                                 
     default: return(SellLotCount_(Money_Management, 
                                       Margin_Mode, STOPLOSS));
    }
    
    LotA = SellLotCount_(Money_Management, MarModeA, STOPLOSS);
    if (LotA == -1)
              return(-1);
    //----          
    LotB = SellLotCount_(Money_Management, MarModeB, STOPLOSS);
    if (LotB == -1)
              return(-1);
   //----              
   if (LotA < LotB)
        return(LotA);
   else return(LotB);         
//---+
  }                                
//+==================================================================+
//| Custom Expert initialization function                            |
//+==================================================================+
int init()
  {
//----+ 
    if (NewsHour > 23) 
             NewsHour = 23;
    //----         
    if (NewsMinute > 59)
           NewsMinute = 59;                               
    //---- Expert Advisor initialization complete
   return(0);
//----+
  }
//+X================================================================X+
//| expert deinitialization function                                 |
//+X================================================================X+ 
int deinit()
  {
//----+
    //---- Deleting global variables of the test mode
    TimeLevelGlobalVariableDel(Symbol(), 1);
    TimeLevelGlobalVariableDel(Symbol(), 2);
    //---- Expert Advisor deinitialization complete
    return(0);
//----+ 
  }
//+==================================================================+
//| Custom Expert iteration function                                 |
//+==================================================================+
int start()
  {
//----+
   //----+ Declaration of static variables
   //----+ +----------------------------------------+
   static bool BUY_Sign, SELL_Sign;
   static datetime StopTime, PosStopTime, TimeLevel;
   //----+ +----------------------------------------+
   //---- Declaration of local variables
   bool BUY_Stop, SELL_Stop;
   bool TrailSignal_Up, TrailSignal_Dn;
   
   //---- Getting a signal from the global variable to  
                 // close a long position as soon as the TimeLevel time limit is reached
   if (ClosePos_Up)
     if (TradeTimeLevelCheck(Symbol(), 1, 777))
                                      BUY_Stop = true;                                 
   //---- Getting a signal from the global variable to  
                 // close a short position as soon as the TimeLevel time limit is reached
   if (ClosePos_Dn)
     if (TradeTimeLevelCheck(Symbol(), 2, 777))
                                     SELL_Stop = true; 
                                     
   //---- Trailing Stop for long positions works on every tick!!!
   if (TRAILINGSTOP_Up > 0)
        TrailSignal_Up = true;
   //---- Trailing Stop for short positions works on every tick!!!
   if (TRAILINGSTOP_Dn > 0)
        TrailSignal_Dn = true;
   //---- Getting the time to disable repeated Trailing Stop.
                              // 0 - Trailing Stop works on every tick!!!
   datetime TrailTimeLevel = 0;
      
   //----+ DETERMINING SIGNALS FOR DEALS 
   if (DayOfWeek() == NewsWeekDay 
     || NewsWeekDay < 1 
       || NewsWeekDay > 5)
         if (Hour() == NewsHour)
          if (Minute() == NewsMinute)
    {
     //---- getting the expiration time of pending orders
     StopTime = TimeCurrent() + OrderLife * 60;
     //---- Getting the time to disable repeated deal 
                                      // and the position closing time
     TimeLevel = TimeCurrent() + PosLife * 60;
     //----
     BUY_Sign = true;
     SELL_Sign = true;
    }
   
   //---- Deleting signals for execution of deals after
                           // the expiration time of pending orders
   if (TimeCurrent() >= StopTime)
    {
     BUY_Sign = false;
     SELL_Sign = false;
    }

   //----+ +---------------------------------------------------------------+ 
   //----+ CODE FOR LONG POSITIONS
   if (Test_Up) 
    {
      //----+ DEAL EXECUTION          
      if (!CloseBuyOrder1_(BUY_Stop, 1))
                                    return(-1);
      //----                              
      if (!OpenBuyStopOrder1_(BUY_Sign, 1, TimeLevel, 
                Money_Management_Up, 0, STOPLOSS_Up, TAKEPROFIT_Up,
                                             PriceLevel_Up, StopTime))
                                                               return(-1);   
      //----                                     
      if (!Make_BuyTrailingStop_(TrailSignal_Up, 1, 
                        TrailTimeLevel, TRAILINGSTOP_Up))
                                                   return(-1); 
     }
   //----+ +---------------------------------------------------------------+
   //----+ CODE FOR SHORT POSITIONS
   if (Test_Dn) 
     {
      //----+ DEAL EXECUTION
      if (!CloseSellOrder1_(SELL_Stop, 2))
                                     return(-1);
      //----                               
      if (!OpenSellStopOrder1_(SELL_Sign, 2, TimeLevel, 
                Money_Management_Dn, 0, STOPLOSS_Dn, TAKEPROFIT_Dn,
                                             PriceLevel_Dn, StopTime))
                                                                return(-1);
      //----                                  
      if (!Make_SellTrailingStop_(TrailSignal_Dn, 2, 
                        TrailTimeLevel, TRAILINGSTOP_Dn))
                                                   return(-1);
     }
   //----+ +---------------------------------------------------------------+
   
   return(0);
//----+
  }
//+------------------------------------------------------------------+